/*
 catalogo.sql - Archivo de creacin de la base de datos para el portal Inmuebles.

 Autor: Ricardo de los Ros
 Proyecto: TFC J2EE
 UOC - Junio 2011

*/

-- Borrado de tablas previas

drop table fotos;
drop table denuncia;
drop table factura;
drop table anuncio;
drop table busqueda;
drop table inmueble;
drop table distrito;
drop table provincia;
drop table pais;
drop table usuario;

-- Borrado de secuencias previas

drop sequence secuencia_idUsuario;
drop sequence secuencia_idProvincia;
drop sequence secuencia_idPais;
drop sequence secuencia_idDistrito;
drop sequence secuencia_idInmueble;
drop sequence secuencia_idAnuncio;
drop sequence secuencia_idDenuncia;
drop sequence secuencia_idFactura;
drop sequence secuencia_idBusqueda;
drop sequence secuencia_idFotos;

-- Definicin de las tablas, restricciones y relaciones.

CREATE TABLE usuario (
idUsuario NUMBER(12) CONSTRAINT PK_usuario PRIMARY KEY,
tipoUsuario CHAR(1) CONSTRAINT NN_tipoUsuario NOT NULL CONSTRAINT CH_tipoUsuario CHECK(tipoUsuario IN ('N','D','P')),
correo VARCHAR2(255) CONSTRAINT NN_correo NOT NULL CONSTRAINT UN_correo UNIQUE,
password VARCHAR2(20) CONSTRAINT NN_password NOT NULL,
nombre VARCHAR2(30) CONSTRAINT NN_nombre NOT NULL,
apellidos VARCHAR2(50) CONSTRAINT NN_apellidos NOT NULL,
telefono1 VARCHAR2(20) CONSTRAINT NN_telefono1 NOT NULL,
telefono2 VARCHAR2(20)
);

CREATE TABLE pais (
idPais NUMBER(12) CONSTRAINT PK_pais PRIMARY KEY,
nombre VARCHAR2(20) CONSTRAINT NN_nombrePais NOT NULL CONSTRAINT UN_nombrePais UNIQUE,
geolocalizacion VARCHAR2(255)
);

CREATE TABLE provincia (
idprovincia NUMBER(12) CONSTRAINT PK_provincia PRIMARY KEY,
nombre VARCHAR2(20) CONSTRAINT NN_nombreProvincia NOT NULL CONSTRAINT UN_nombreProvincia UNIQUE,
pais NUMBER(12) CONSTRAINT FK_paisProvincia REFERENCES pais(idPais) CONSTRAINT NN_fkPaisProvincia NOT NULL,
geolocalizacion VARCHAR2(255)
);

CREATE TABLE distrito (
idDistrito NUMBER(12) CONSTRAINT PK_distrito PRIMARY KEY,
nombre VARCHAR2(20) CONSTRAINT NN_nombreDistrito NOT NULL CONSTRAINT UN_nombreDistrito UNIQUE,
provincia NUMBER(12) CONSTRAINT FK_provinciaDistrito REFERENCES provincia(idprovincia) CONSTRAINT NN_fkprovinciaDistrito NOT NULL,
geolocalizacion VARCHAR2(255)
);

CREATE TABLE inmueble (
idInmueble NUMBER(12) CONSTRAINT PK_inmueble PRIMARY KEY,
tipoPiso CHAR(1) CONSTRAINT NN_tipoPiso NOT NULL CONSTRAINT CH_tipoPiso CHECK(tipoPiso IN ('V','H','O','L','G')),
superficie NUMBER(5,1) CONSTRAINT CH_superficie CHECK (superficie>0),
direccion VARCHAR2(250) CONSTRAINT NN_direccion NOT NULL,
codigoPostal VARCHAR(10),
orientacion CHAR(1),
geolocalizacion VARCHAR2(250),
numDormitorios NUMBER(2) CONSTRAINT CH_numDormitorios CHECK (numDormitorios>-1),
numBanios NUMBER(2) CONSTRAINT CH_numBanios CHECK (numBanios>-1),
ascensor CHAR(1) CONSTRAINT CH_ascensor CHECK (ascensor IN ('S', 'N')),
trastero CHAR(1) CONSTRAINT CH_trastero CHECK (trastero IN ('S', 'N')),
terraza CHAR(1) CONSTRAINT CH_terraza CHECK (terraza IN ('S', 'N')),
piscina CHAR(1) CONSTRAINT CH_piscina CHECK (piscina IN ('S', 'N')),
aguaCalienteIndv CHAR(1) CONSTRAINT CH_aguaCalienteIndv CHECK (aguaCalienteIndv IN ('S', 'N')),
calefaccionIndv CHAR(1) CONSTRAINT CH_calefaccionIndv CHECK (calefaccionIndv IN ('S', 'N')),
exterior CHAR(1) CONSTRAINT CH_exterior CHECK (exterior IN ('S', 'N')),
pais NUMBER(12) CONSTRAINT FK_paisInmueble REFERENCES pais(idPais) CONSTRAINT NN_fkpaisInmueble NOT NULL,
provincia NUMBER(12) CONSTRAINT FK_provinciaInmueble REFERENCES provincia(idProvincia) CONSTRAINT NN_fkprovinciaInmueble NOT NULL,
distrito NUMBER(12) CONSTRAINT FK_distritoInmueble REFERENCES distrito(idDistrito) CONSTRAINT NN_fkdistritoInmueble NOT NULL
);

CREATE TABLE anuncio (
idAnuncio NUMBER(12) CONSTRAINT PK_idAnuncio  PRIMARY KEY,
tipoAccion CHAR(1) CONSTRAINT NN_tipoAccion NOT NULL CONSTRAINT CH_tipoAccion CHECK(tipoAccion IN ('C','A','O')),
fechaPublicacion DATE DEFAULT SYSDATE CONSTRAINT NN_fechaPublicacion NOT NULL,
descripcion VARCHAR2(250),
precio NUMBER(10) CHECK (precio>-1),
estado CHAR(1) CONSTRAINT CH_estado CHECK(estado IN ('P','E','R')),
alquilerGarantizado CHAR(1) CONSTRAINT CH_alquilerGarantizado CHECK (alquilerGarantizado IN ('S', 'N')),
inmueble NUMBER(12) CONSTRAINT FK_inmuebleAnuncio REFERENCES inmueble(idInmueble) CONSTRAINT NN_fkinmuebleAnuncio NOT NULL,
idUsuario NUMBER(12) CONSTRAINT FK_usuarioAnuncio REFERENCES usuario(idUsuario) CONSTRAINT NN_fkusuarioAnuncio NOT NULL
);

CREATE TABLE Denuncia (
idDenuncia NUMBER(12) CONSTRAINT PK_idDenuncia PRIMARY KEY,
descripcion VARCHAR2(250) CONSTRAINT NN_descripcionDenuncia NOT NULL,
anuncio NUMBER(12) CONSTRAINT FK_anuncioDenuncia REFERENCES anuncio(idAnuncio) CONSTRAINT NN_fkanuncioDenuncia NOT NULL
);

CREATE TABLE Factura (
idFactura NUMBER(12) CONSTRAINT PK_idFactura PRIMARY KEY,
fechaCompra DATE DEFAULT SYSDATE CONSTRAINT NN_fechaCompra NOT NULL,
periodoValidez NUMBER(10) CHECK (periodoValidez>-1),
nombreProducto VARCHAR2(100) CONSTRAINT NN_nombreProducto NOT NULL,
usuario NUMBER(12) CONSTRAINT FK_usuarioFactura REFERENCES usuario(idUsuario) CONSTRAINT NN_fkusuarioFactura NOT NULL
);

CREATE TABLE busqueda (
idBusqueda NUMBER(12) CONSTRAINT PK_busqueda PRIMARY KEY,
inmueble NUMBER(12) CONSTRAINT FK_inmuebleBusqueda REFERENCES inmueble(idInmueble) CONSTRAINT NN_fkinmuebleBusqueda NOT NULL,
usuario NUMBER(12) CONSTRAINT FK_usuarioBusqueda REFERENCES usuario(idUsuario) CONSTRAINT NN_fkusuarioBusqueda NOT NULL,
tipoPiso CHAR(6) CONSTRAINT NN_tipoPisoBusqueda NOT NULL,
minSuperficie NUMBER(5,1) CHECK (minSuperficie>0),
maxSuperficie NUMBER(5,1) CHECK (maxSuperficie>0),
precioMin NUMBER(10) CHECK (precioMin>-1),
precioMax NUMBER(10) CHECK (precioMax>-1),
estado CHAR(3) CONSTRAINT NN_estado NOT NULL,
maxNumDorm NUMBER(2) CONSTRAINT CH_maxNumDorm CHECK (maxNumDorm>-1),
maxNumBanios NUMBER(2) CONSTRAINT CH_maxNumBanios CHECK (maxNumBanios>-1),
conAscensor CHAR(1) CONSTRAINT CH_conAscensor CHECK (conAscensor IN ('S', 'N','I')),
conTrastero CHAR(1) CONSTRAINT CH_conTrastero CHECK (conTrastero IN ('S', 'N','I')),
conTerraza CHAR(1) CONSTRAINT CH_conTerraza CHECK (conTerraza IN ('S', 'N','I')),
conPiscina CHAR(1) CONSTRAINT CH_conPiscina CHECK (conPiscina IN ('S', 'N','I')),
conAguaCalienteIndv CHAR(1) CONSTRAINT CH_conAguaCalienteIndv CHECK (conAguaCalienteIndv IN ('S', 'N','I')),
conCalefaccionIndv CHAR(1) CONSTRAINT CH_conCalefaccionIndv CHECK (conCalefaccionIndv IN ('S', 'N','I')),
conExterior CHAR(1) CONSTRAINT CH_conExterior CHECK (conExterior IN ('S', 'N','I'))
);

CREATE TABLE Fotos (
idFoto NUMBER(12) CONSTRAINT PK_idFoto PRIMARY KEY,
descripcion VARCHAR2(250),
inmueble NUMBER(12) CONSTRAINT FK_inmuebleFoto REFERENCES inmueble(idInmueble) CONSTRAINT NN_fkinmuebleFoto NOT NULL,
foto BLOB CONSTRAINT NN_foto NOT NULL
);

-- Creacin de secuencias para autoincrementales

create sequence secuencia_idUsuario
   start with 1 
   increment by 1 
   nomaxvalue;

create or replace trigger autoincrement_idUsuario
before insert on usuario for each row
begin
    select secuencia_idUsuario.nextval 
    into :new.idUsuario
    from dual;
end autoincrement_idUsuario;
/

create sequence secuencia_idProvincia
   start with 1 
   increment by 1 
   nomaxvalue;

create or replace trigger autoincrement_idProvincia
before insert on provincia for each row
begin
    select secuencia_idProvincia.nextval 
    into :new.idProvincia
    from dual;
end autoincrement_idProvincia;
/

create sequence secuencia_idDistrito
   start with 1 
   increment by 1 
   nomaxvalue;

create or replace trigger autoincrement_idDistrito
before insert on distrito for each row
begin
    select secuencia_idDistrito.nextval 
    into :new.idDistrito
    from dual;
end autoincrement_idDistrito;
/

create sequence secuencia_idInmueble
   start with 1 
   increment by 1 
   nomaxvalue;

create or replace trigger autoincrement_idInmueble
before insert on inmueble for each row
begin
    select secuencia_idInmueble.nextval 
    into :new.idInmueble
    from dual;
end autoincrement_idInmueble;
/

create sequence secuencia_idAnuncio
   start with 1 
   increment by 1 
   nomaxvalue;

create or replace trigger autoincrement_idAnuncio
before insert on anuncio for each row
begin
    select secuencia_idAnuncio.nextval 
    into :new.idAnuncio
    from dual;
end autoincrement_idAnuncio;
/

create sequence secuencia_idDenuncia
   start with 1 
   increment by 1 
   nomaxvalue;

create or replace trigger autoincrement_idDenuncia
before insert on denuncia for each row
begin
    select secuencia_idDenuncia.nextval 
    into :new.idDenuncia
    from dual;
end autoincrement_idDenuncia;
/

create sequence secuencia_idFactura
   start with 1 
   increment by 1 
   nomaxvalue;

create or replace trigger autoincrement_idFactura
before insert on factura for each row
begin
    select secuencia_idFactura.nextval 
    into :new.idFactura
    from dual;
end autoincrement_idFactura;
/

create sequence secuencia_idBusqueda
   start with 1 
   increment by 1 
   nomaxvalue;

create or replace trigger autoincrement_idBusqueda
before insert on busqueda for each row
begin
    select secuencia_idBusqueda.nextval 
    into :new.idBusqueda
    from dual;
end autoincrement_idBusqueda;
/

create sequence secuencia_idFotos
   start with 1 
   increment by 1 
   nomaxvalue;

create or replace trigger autoincrement_idFotos
before insert on fotos for each row
begin
    select secuencia_idFotos.nextval 
    into :new.idFoto
    from dual;
end autoincrement_idFotos;
/

create sequence secuencia_idPais
   start with 1 
   increment by 1 
   nomaxvalue;

create or replace trigger autoincrement_idPais
before insert on pais for each row
begin
    select secuencia_idPais.nextval 
    into :new.idPais
    from dual;
end autoincrement_idPais;
/

commit;