package inmuebles.modelos;

import java.sql.Connection; 
import java.sql.PreparedStatement; 
import java.sql.ResultSet; 
import java.sql.SQLException; 

import java.util.ArrayList; 
import java.util.List; 
import inmuebles.beans.Anuncio;
import inmuebles.util.ExcepcionInmuebles;

/**
 * Clase DAO que representa un anuncio de inmueble. 
 * 
 * Implementa métodos diversos de acceso a base de datos para la entidad anuncio que posibilita
 * obtener listados de anuncios, realizar inserciones de nuevos anuncios, eliminar anuncios etc.
 * 
 * @author Ricardo de los Rios
 * @see ModeloDAO
 */

public class ModeloAnuncioDAO extends ModeloDAO{

       // Variables 
       Connection conexion=null; 
       ResultSet resultado=null; 
       private static List<Anuncio> listaAnuncios;

/**
 *  Método para obtener la lista de anuncios de un usuario.
 * 
 * @return Lista de anuncios.
 */   
        
       // devolver la lista de anuncios
       public List<Anuncio> getListaAnuncios(int idUsuario) 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              Anuncio anuncio=null; 
              String consultaString=null; 
              listaAnuncios=new ArrayList<Anuncio>();

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // consulta de lista de anuncios 
                    //consultaString="SELECT * FROM anuncio where idusuario=?";
                    consultaString = "SELECT * FROM anuncio a, inmueble i WHERE a.inmueble=i.idinmueble and a.idusuario=?";

                    consulta=conexion.prepareStatement(consultaString);
                    consulta.setInt(1, idUsuario);

                     // Ejecución de la consulta 

                    resultado=consulta.executeQuery();


                     // Se almacena el resultado en una lista 
                     if(resultado!=null) 
                     { 
                            while(resultado.next()) 
                            { 
                                   // Se efectua el mapping de los atributos con los campos de la tabla SQL 
                                   anuncio=mapperAnuncio(resultado);

                                   // Se aniade el objeto a la lista de anuncios

                                    listaAnuncios.add((Anuncio)anuncio); 
                            } 
                     } 
              } 
              catch(Exception e) 
              { 
                     System.out.println("Error en la consulta de la clase ModeloAnuncioDAO funcion getListaAnuncios"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                                  
                            } 
                            if(consulta!=null) 
                            {

                                   consulta.close();
                            } 
                            if(conexion!=null) 
                            {

                                conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexion con la base de datos en la clase ModeloAnuncioDAO funcion getListaAnuncios"); 
                     } 
              }

              // Devolver la lista de anuncios 
              return listaAnuncios; 
       }
       
  /**
 *  Método para obtener la lista de anuncios que corresponde a un filtro de búsqueda.
 * 
 * @return Lista de anuncios.
 */   
               // devolver la lista de anuncios
       public List<Anuncio> getListaAnunciosFiltro(String queryFiltro) 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              Anuncio anuncio=null; 
              String consultaString=null; 
              listaAnuncios=new ArrayList<Anuncio>();

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // consulta de lista de anuncios 
                    consultaString="SELECT * FROM anuncio a, inmueble i where a.inmueble=i.idInmueble " + queryFiltro;

                    consulta=conexion.prepareStatement(consultaString);             

                     // Ejecución de la consulta 

                    resultado=consulta.executeQuery();


                     // Se almacena el resultado en una lista 
                     if(resultado!=null) 
                     { 
                            while(resultado.next()) 
                            { 
                                   // Se efectua el mapping de los atributos con los campos de la tabla SQL 
                                   anuncio=mapperAnuncio(resultado);

                                   // Se aniade el objeto a la lista de anuncios

                                    listaAnuncios.add((Anuncio)anuncio); 
                            } 
                     } 
              } 
              catch(Exception e) 
              { 
                     System.out.println("Error en la consulta de la clase ModeloAnuncioDAO funcion getListaAnuncios"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                                  
                            } 
                            if(consulta!=null) 
                            {

                                   consulta.close();
                            } 
                            if(conexion!=null) 
                            {

                                conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexion con la base de datos en la clase ModeloAnuncioDAO funcion getListaAnuncios"); 
                     } 
              }

              // Devolver la lista de anuncios 
              return listaAnuncios; 
       }

 /**
 *  Método para obtener un anuncio dado su código.
 * 
 * @param idAnuncio Código del anuncio.
 * @return Anuncio o null si no existe para ese código.
 */   
 
       public Anuncio getAnuncio(String idAnuncio) 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              Anuncio anuncio=null; 
              String consultaString=null;

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // Creacion de la consulta 
                   consultaString = "SELECT * FROM anuncio a, inmueble i WHERE a.inmueble=i.idinmueble and a.idAnuncio=?";

                     // Se prepara la consulta

                     consulta=conexion.prepareStatement(consultaString); 
                     consulta.setString(1,idAnuncio);

                     // Ejecucion de la consulta 
                     resultado=consulta.executeQuery();

                     // Se almacena el resultado en el objeto anuncio 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                   // Se realiza el mapping de los atributos con los campos de la tabla SQL 
                                  anuncio=mapperAnuncio(resultado); 
                            } 
                     } 
              } 
              catch(Exception e) 
              { 
                     anuncio=null; 
                     System.out.println("Error en la consulta en la clase ModeloAnuncioDAO funcion getAnuncio"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                            } 
                            if(consulta!=null) 
                            {
                                consulta.close();
                            } 
                            if(conexion!=null) 
                            {
                               conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexión con la base de datos en la clase ModeloAnuncioDAO función getAnuncio"); 
                     } 
              }

              // Devolver objeto anuncio 
              return anuncio; 
       }      

 /**
 *  Método para agregar un nuevo anuncio a la base de datos.
 * 
 * @param anuncio Información del anuncio.
 */   

       public void agregarAnuncio(Anuncio anuncio) throws ExcepcionInmuebles 
       { 
              // Variables 
              PreparedStatement consulta1=null; 
              PreparedStatement consulta2=null; 
              PreparedStatement consulta3=null; 
              PreparedStatement consulta4=null; 

              String consultaString=null; 
              int codigoError=0;
              String idinmueble="";
              String idanuncio="";

              try 
              { 
                     // Apertura de una conexión 
                     conexion=super.getConnection();

                     // Creación del nuevo en inmueble
                     consultaString="INSERT INTO inmueble (tipopiso,superficie, distrito,direccion, codigopostal,"
                             + " orientacion,  numDormitorios, numbanios,"
                             + "ascensor,trastero, terraza, piscina, aguacalienteindv,calefaccionindv, exterior,pais,provincia)"
                             + " VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

                     consulta1=conexion.prepareStatement(consultaString); 
                     consulta1.setString(1, anuncio.getTipo_inmueble());
                     consulta1.setString(2, anuncio.getSuperficie());
                     consulta1.setString(3, anuncio.getDistrito());
                     consulta1.setString(4, anuncio.getDireccion());
                     consulta1.setString(5, anuncio.getCodigo_postal());
                     consulta1.setString(6, anuncio.getOrientacion());
                     consulta1.setString(7, anuncio.getNum_dorms());
                     consulta1.setString(8, anuncio.getNum_banios());
                     consulta1.setString(9, anuncio.getAscensor());
                     consulta1.setString(10, anuncio.getTrastero());
                     consulta1.setString(11, anuncio.getTerraza());
                     consulta1.setString(12, anuncio.getPiscina());
                     consulta1.setString(13, anuncio.getAgua_caliente());
                     consulta1.setString(14, anuncio.getCalefaccion_indv());
                     consulta1.setString(15, anuncio.getExterior());
                     consulta1.setString(16, "1");
                    consulta1.setString(17, anuncio.getProvincia());

                     codigoError=consulta1.executeUpdate();                     

                                          // Recupera el nuevo codigo de inmueble
                     consultaString="select Max(idinmueble) from inmueble";
                     
                     consulta2=conexion.prepareStatement(consultaString);

                    resultado=consulta2.executeQuery();
 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                   // Se realiza el mapping de los atributos con los campos de la tabla SQL 
                                  idinmueble=resultado.getString(1); 
                            } 
                     } 
                     
                     // Creación de la consulta en anuncio
                     consultaString="INSERT INTO anuncio (inmueble,idusuario, tipoaccion,fechapublicacion, descripcion,"
                             + " precio, estado, alquilergarantizado) VALUES(?,?,?,?,?,?,?,?)";
                     System.out.println(consultaString);

                     consulta3=conexion.prepareStatement(consultaString); 
                     consulta3.setString(1, idinmueble);
                     consulta3.setString(2, anuncio.getIdUsuario());
                     consulta3.setString(3, anuncio.getTipoAccion());
                      
                     anuncio.setFecha_publicacion("20/5/2011");
                     
                     consulta3.setString(4, anuncio.getFecha_publicacion());
                     consulta3.setString(5, anuncio.getDescripcion());
                     consulta3.setString(6, anuncio.getPrecio());
                     consulta3.setString(7, anuncio.getEstado());
                     consulta3.setString(8, anuncio.getAlquiler_garantizado());

                     codigoError=consulta3.executeUpdate(); 

                 // Recupera el nuevo codigo de anuncio
                     consultaString="select Max(idanuncio) from anuncio";
                    consulta4=conexion.prepareStatement(consultaString);
                    resultado=consulta4.executeQuery();
 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                   // Se realiza el mapping de los atributos con los campos de la tabla SQL 
                                  idanuncio=resultado.getString(1); 
                                  anuncio.setIdAnuncio(idanuncio);
                            } 
                     } 

              } 
              catch(SQLException esql) 
              { 
                     
                     System.out.println("Error SQL en la consulta de la clase ModeloAnuncioDAO función agregarAnuncio" +
                             "\n Flag de executeUpdate: " + codigoError +
                             "\n Codigo de error SQL: " + esql.getErrorCode() +
                             "\n Mensaje de error: " + esql.getMessage() +
                             "\n Estado SQL del error: " + esql.getSQLState()); 
                    
                     throw new ExcepcionInmuebles(ExcepcionInmuebles.EXCEPCION_FUNCIONAL,esql.getMessage()); 
                  
              } 
              catch(Exception e) 
              { 
                  String ra="Error genérico en la consulta de la clase ModeloUsuarioDAO función agregarUsuario" +
                             "\n Codigo de error: " + codigoError +
                             "\n Mensaje de error: " + e.getMessage();          
                  
                  throw new ExcepcionInmuebles(ExcepcionInmuebles.EXCEPCION_TECNICA,ra); 
              }
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                            } 
                            if(consulta1!=null) 
                            {                             
                                consulta1.close(); 
                            } 
                            if(consulta2!=null) 
                            {                             
                                consulta2.close(); 
                            } 
                            if(consulta3!=null)  { consulta3.close(); } 
                            if(consulta4!=null) {  consulta4.close(); } 
                            if(conexion!=null) { conexion.close(); } 

                     } 
                     catch(Exception ex) 
                     { 
                            String ra="Error en el cierre de la conexion con la base de datos en la clase ModeloUsuarioDAO funcion agregarUsuario";                            
                            ra+=ex.getMessage();
                            
                            throw new ExcepcionInmuebles(ExcepcionInmuebles.EXCEPCION_TECNICA,ra);                      } 
                     
                       }
              }

            
    

/**
 *  Método para mapear la información de un anuncio desde un objeto ResultSet de SQL al bean java.
 * 
 * @param resultado ResultSet con información del anuncio
 * @return Bean de java rellenado con la información
 */   
    
    // Realizar el mapping relacional hacia objeto 
    public Anuncio mapperAnuncio(ResultSet resultado) 
    { 
       // Variables 
       Anuncio anuncio=new Anuncio();

       try 
       { 
              if (resultado.getString("idAnuncio")==null) 
              { 
                     anuncio.setIdAnuncio("0"); 
              } 
              else 
              {
                anuncio.setIdAnuncio(resultado.getString("idAnuncio"));
              }
              
              if (resultado.getString("tipoAccion")==null) 
              { 
                     anuncio.setIdAnuncio("0"); 
              } 
              else 
              {
                anuncio.setTipoAccion(resultado.getString("tipoAccion"));
              }

              if (resultado.getString("fechaPublicacion")==null) 
                anuncio.setFecha_publicacion("");        
              else anuncio.setFecha_publicacion(resultado.getString("fechaPublicacion"));
              
              if (resultado.getString("descripcion")==null) 
              { 
                     anuncio.setIdAnuncio(""); 
              } 
              else 
              {
                anuncio.setDescripcion(resultado.getString("descripcion"));
              }

              if (resultado.getString("precio")==null) 
              { 
                     anuncio.setPrecio(""); 
              } 
              else 
              {
                anuncio.setPrecio(resultado.getString("precio"));
              }
              
              if (resultado.getString("direccion")==null) 
                anuncio.setDireccion("");        
              else anuncio.setDireccion(resultado.getString("direccion"));
              
              if (resultado.getString("estado")==null) 
                anuncio.setEstado("");        
              else anuncio.setEstado(resultado.getString("estado"));

              if (resultado.getString("alquilerGarantizado")==null) 
                anuncio.setAlquiler_garantizado("");        
              else anuncio.setAlquiler_garantizado(resultado.getString("alquilerGarantizado"));
              
               if (resultado.getString("superficie")==null) 
                anuncio.setSuperficie("");        
              else anuncio.setSuperficie(resultado.getString("superficie"));

              if (resultado.getString("inmueble")==null) 
                anuncio.setIdInmueble("0");        
              else anuncio.setIdInmueble(resultado.getString("inmueble"));

              if (resultado.getString("idUsuario")==null) 
                anuncio.setIdUsuario("0");        
              else anuncio.setIdUsuario(resultado.getString("idUsuario"));
              
               if (resultado.getString("ascensor")==null) 
                anuncio.setAscensor("");        
              else anuncio.setAscensor(resultado.getString("ascensor"));
               
                if (resultado.getString("piscina")==null) 
                anuncio.setPiscina("");        
              else anuncio.setPiscina(resultado.getString("piscina"));
              
                  if (resultado.getString("trastero")==null) 
                anuncio.setTrastero("");        
              else anuncio.setTrastero(resultado.getString("trastero"));
                  
               if (resultado.getString("codigopostal")==null) 
                anuncio.setCodigo_postal("");        
              else anuncio.setCodigo_postal(resultado.getString("codigopostal"));

                if (resultado.getString("orientacion")==null) 
                anuncio.setOrientacion("");        
              else anuncio.setOrientacion(resultado.getString("orientacion"));

               if (resultado.getString("numdormitorios")==null) 
                anuncio.setNum_dorms("");        
              else anuncio.setNum_dorms(resultado.getString("numdormitorios"));

              if (resultado.getString("numBanios")==null) 
                anuncio.setNum_banios("");        
              else anuncio.setNum_banios(resultado.getString("numBanios"));
              
              if (resultado.getString("calefaccionIndv")==null) 
                anuncio.setCalefaccion_indv("");        
              else anuncio.setCalefaccion_indv(resultado.getString("calefaccionIndv"));

               if (resultado.getString("aguaCalienteIndv")==null) 
                anuncio.setAgua_caliente("");        
              else anuncio.setAgua_caliente(resultado.getString("aguaCalienteIndv"));

              
              if (resultado.getString("exterior")==null) 
                anuncio.setExterior("");        
              else anuncio.setExterior(resultado.getString("exterior"));

       } 
        catch (Exception e) 
        { 
              //Si se produce un error durante el mapping de atributos 
              System.out.println("Error en el mapping de atributos de un usuario de la clase ModeloUsuarioDAO, funcion mapperUsuario"); 
        }

        // Devolver objeto usuario 
        return anuncio; 
    }

       // eliminar un anuncio en la base 
       public int eliminarAnuncio(int idAnuncio) 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              String consultaString=null; 
              int codigoError=0;

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // Eliminar el anuncio 
                     consultaString="DELETE FROM anuncio WHERE idAnuncio=?";

       consulta=conexion.prepareStatement(consultaString); 
                     consulta.setInt(1,idAnuncio);

                     // Ejecucion de la consulta 
                     codigoError=consulta.executeUpdate(); 
              } 
              catch(Exception e) 
              { 
                     codigoError=0; 
                     System.out.println("Error en la consulta de la clase ModeloAnuncioDAO funcion eliminarAnuncio"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                
                                   resultado.close();
                            } 
                            if(consulta!=null) 
                            {

                                    consulta.close();
                            } 
                            if(conexion!=null) 
                            {

                                conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexion con la base de datos en la clase ModeloAnuncioDAO funcion eliminarAnuncio"); 
                      } 
              }

              // Devolver el codigo de error 
              return codigoError; 
       }
 
}