package inmuebles.modelos;

import java.sql.Connection; 
import java.sql.PreparedStatement; 
import java.sql.ResultSet; 
import java.sql.SQLException; 

import java.util.ArrayList; 
import java.util.List; 
import inmuebles.beans.Distrito;

import inmuebles.util.ExcepcionInmuebles;

/**
 * Clase DAO que obtiene información Distritos. 
 * 
 * 
 * @author Ricardo de los Rios
 * @see ModeloDAO
 */

public class ModeloDistritoDAO extends ModeloDAO{

       // Variables 
       Connection conexion=null; 
       ResultSet resultado=null; 
       private static List<Distrito> listaDistritos;

/**
 *  Método para obtener la lista de Distritos.
 * 
 * @return Lista de Distritos.
 */   
        
       // devolver la lista de Distritos
       public List<Distrito> getListaDistritos() 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              Distrito Distrito=null; 
              String consultaString=null; 
              listaDistritos=new ArrayList<Distrito>();

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // consulta de lista de Distritos 
                    consultaString="SELECT * FROM Distrito";

                    consulta=conexion.prepareStatement(consultaString);
                    
                     // Ejecución de la consulta 

                    resultado=consulta.executeQuery();


                     // Se almacena el resultado en una lista 
                     if(resultado!=null) 
                     { 
                            while(resultado.next()) 
                            { 
                                   // Se efectua el mapping de los atributos con los campos de la tabla SQL 
                                   Distrito=mapperDistrito(resultado);

                                   // Se aniade el objeto a la lista de Distritos

                                    listaDistritos.add((Distrito)Distrito); 
                            } 
                     } 
              } 
              catch(Exception e) 
              { 
                     System.out.println("Error en la consulta de la clase ModeloDistritoDAO funcion getListaDistritos"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                                  
                            } 
                            if(consulta!=null) 
                            {

                                   consulta.close();
                            } 
                            if(conexion!=null) 
                            {

                                conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexion con la base de datos en la clase ModeloDistritoDAO funcion getListaDistritos"); 
                     } 
              }

              // Devolver la lista de Distritos 
              return listaDistritos; 
       }

 /**
 *  Método para obtener un Distrito dado su código.
 * 
 * @param idDistrito Código del Distrito.
 * @return Distrito o null si no existe para ese código.
 */   
 
       public Distrito getDistrito(int idDistrito) 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              Distrito Distrito=null; 
              String consultaString=null;

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // Creacion de la consulta 
                     consultaString = "SELECT * FROM Distrito WHERE idDistrito=?";

                     // Se prepara la consulta

                     consulta=conexion.prepareStatement(consultaString); 
                     consulta.setInt(1,idDistrito);

                     // Ejecucion de la consulta 
                     resultado=consulta.executeQuery();

                     // Se almacena el resultado en el objeto Distrito 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                   // Se realiza el mapping de los atributos con los campos de la tabla SQL 
                                  Distrito=mapperDistrito(resultado); 
                            } 
                     } 
              } 
              catch(Exception e) 
              { 
                     Distrito=null; 
                     System.out.println("Error en la consulta en la clase ModeloDistritoDAO funcion getDistrito"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                            } 
                            if(consulta!=null) 
                            {
                                consulta.close();
                            } 
                            if(conexion!=null) 
                            {
                               conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexión con la base de datos en la clase ModeloDistritoDAO función getDistrito"); 
                     } 
              }

              // Devolver objeto Distrito 
              return Distrito; 
       }      

    

/**
 *  Método para mapear la información de un Distrito desde un objeto ResultSet de SQL al bean java.
 * 
 * @param resultado ResultSet con información del Distrito
 * @return Bean de java rellenado con la información
 */   
    
    // Realizar el mapping relacional hacia objeto 
    public Distrito mapperDistrito(ResultSet resultado) 
    { 
       // Variables 
       Distrito Distrito=new Distrito();

       try 
       { 
              if (resultado.getString("idDistrito")==null) 
              { 
                     Distrito.setIdDistrito("0"); 
              } 
              else 
              {
                Distrito.setIdDistrito(resultado.getString("idDistrito"));
              }
              
              if (resultado.getString("nombre")==null) 
              { 
                     Distrito.setIdDistrito(""); 
              } 
              else 
              {
                Distrito.setNombre(resultado.getString("nombre"));
              }
            
              if (resultado.getString("provincia")==null) 
                Distrito.setIdDistrito("0");        
              else Distrito.setIdDistrito(resultado.getString("provincia"));
              
       } 
        catch (Exception e) 
        { 
              //Si se produce un error durante el mapping de atributos 
              System.out.println("Error en el mapping de atributos de un usuario de la clase ModeloUsuarioDAO, funcion mapperUsuario"); 
        }

        // Devolver objeto usuario 
        return Distrito; 
    } 
}