package inmuebles.modelos;

import java.sql.Connection; 
import java.sql.PreparedStatement; 
import java.sql.ResultSet; 
import java.sql.SQLException; 

import inmuebles.beans.EstadisticaAnuncios;
import inmuebles.beans.EstadisticaClientes;
import inmuebles.util.ExcepcionInmuebles;

/**
 * Clase DAO que representa las estadisticas de inmuebles y usuarios del portal. 
 *  
 * @author Ricardo de los Rios
 * @see ModeloDAO
 */

public class ModeloEstadisticaDAO extends ModeloDAO{

       // Variables 
       Connection conexion=null; 
       ResultSet resultado=null; 

 /**
 *  Método para obtener estadisticas de estadisticas.
 * 
 * @return Estadisticas de estadisticas
 */   
 
       public EstadisticaAnuncios getEstadisticaAnuncios() 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              EstadisticaAnuncios estadistica=new EstadisticaAnuncios(); 
              String consultaString=null;

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // Creacion de la consulta 
                   consultaString = "SELECT count(*) recuento FROM anuncio";
                   consulta=conexion.prepareStatement(consultaString); 
                   resultado=consulta.executeQuery();

                     // Se almacena el resultado en el objeto estadistica 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                 estadistica.setNumAnuncios(resultado.getInt("recuento"));
                            } 
                     } 
                     
                   consultaString = "SELECT count(*) recuento FROM anuncio where tipoaccion='C'";
                   consulta=conexion.prepareStatement(consultaString); 
                   resultado=consulta.executeQuery();

                     // Se almacena el resultado en el objeto estadistica 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                 estadistica.setNumAnunciosCompra(resultado.getInt("recuento"));
                            } 
                     } 

                   consultaString = "SELECT count(*) recuento FROM anuncio where tipoaccion='A'";
                   consulta=conexion.prepareStatement(consultaString); 
                   resultado=consulta.executeQuery();

                     // Se almacena el resultado en el objeto estadistica 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                 estadistica.setNumAnunciosAlquiler(resultado.getInt("recuento"));
                            } 
                     } 

                   consultaString = "SELECT count(*) recuento FROM anuncio where tipoaccion='O'";
                   consulta=conexion.prepareStatement(consultaString); 
                   resultado=consulta.executeQuery();

                     // Se almacena el resultado en el objeto estadistica 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                 estadistica.setNumAnunciosCompartir(resultado.getInt("recuento"));
                            } 
                     } 
                     
                     
              } 
              catch(Exception e) 
              { 
                     estadistica=null; 
                     System.out.println("Error en la consulta en la clase ModeloEstadisticaDAO " + e.getMessage()); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                            } 
                            if(consulta!=null) 
                            {
                                consulta.close();
                            } 
                            if(conexion!=null) 
                            {
                               conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexión con la base de datos en la clase ModeloAnuncioDAO función getAnuncio"); 
                     } 
              }

              // Devolver objeto estadistica 
              return estadistica; 
       }      

       /**
 *  Método para obtener estadisticas de estadisticas.
 * 
 * @return Estadisticas de estadisticas
 */   
 
       public EstadisticaClientes getEstadisticaClientes() 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              EstadisticaClientes estadistica=null; 
              String consultaString=null;

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // Creacion de la consulta 
                   consultaString = "SELECT count(*) recuento FROM usuario";

                     // Se prepara la consulta

                     consulta=conexion.prepareStatement(consultaString); 

                     // Ejecucion de la consulta 
                     resultado=consulta.executeQuery();

                     // Se almacena el resultado en el objeto estadistica 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                   // Se realiza el mapping de los atributos con los campos de la tabla SQL 
                                  estadistica=mapperEstadisticasCliente(resultado); 
                            } 
                     } 
                     
                   consultaString = "select count(distinct u.idusuario) recuento from anuncio a , usuario u where a.idusuario=u.idusuario";

                   consulta=conexion.prepareStatement(consultaString); 
                   resultado=consulta.executeQuery();

                     // Se almacena el resultado en el objeto estadistica 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                 estadistica.setMediaAnuncios(resultado.getInt("recuento"));
                            } 
                     } 
                     
              } 
              catch(Exception e) 
              { 
                     estadistica=null; 
                     System.out.println("Error en la consulta en la clase ModeloEstadisticaDAO"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                            } 
                            if(consulta!=null) 
                            {
                                consulta.close();
                            } 
                            if(conexion!=null) 
                            {
                               conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexión con la base de datos en la clase ModeloAnuncioDAO función getAnuncio"); 
                     } 
              }

              // Devolver objeto estadistica 
              return estadistica; 
       }      

 
            
    

/**
 *  Método para mapear la información de un estadistica desde un objeto ResultSet de SQL al bean java.
 * 
 * @param resultado ResultSet con información del estadistica
 * @return Bean de java rellenado con la información
 */   
    
    // Realizar el mapping relacional hacia objeto 
    public EstadisticaClientes mapperEstadisticasCliente(ResultSet resultado) 
    { 
       // Variables 
       EstadisticaClientes estadistica=new EstadisticaClientes();

       try 
       { 
 
              if (resultado.getInt("recuento")==0) 
                estadistica.setNumClientes(0);        
              else estadistica.setNumClientes(resultado.getInt("recuento"));
              
 
       } 
        catch (Exception e) 
        { 
              //Si se produce un error durante el mapping de atributos 
              System.out.println("Error en el mapping de atributos de estadisticas"); 
        }

        // Devolver objeto usuario 
        return estadistica; 
    }
    
    /**
 *  Método para mapear la información de un estadistica desde un objeto ResultSet de SQL al bean java.
 * 
 * @param resultado ResultSet con información del estadistica
 * @return Bean de java rellenado con la información
 */   
    
    // Realizar el mapping relacional hacia objeto 
    public EstadisticaAnuncios mapperEstadisticaAnuncios(ResultSet resultado) 
    { 
       // Variables 
       EstadisticaAnuncios estadistica=new EstadisticaAnuncios();

       try 
       { 
              if (resultado.getInt("recuento")==0) 
                estadistica.setNumAnuncios(0);        
              else estadistica.setNumAnuncios(resultado.getInt("recuento"));
              
 
       } 
        catch (Exception e) 
        { 
              //Si se produce un error durante el mapping de atributos 
              System.out.println("Error en el mapping de atributos de estadisticas"); 
        }

        // Devolver objeto usuario 
        return estadistica; 
    }
 
}