package inmuebles.modelos;

import java.sql.Connection; 
import java.sql.PreparedStatement; 
import java.sql.ResultSet; 
import java.sql.SQLException; 

import java.util.ArrayList; 
import java.util.List; 
import inmuebles.beans.Filtro;
import inmuebles.util.ExcepcionInmuebles;

/**
 * Clase DAO que representa un Filtro de inmueble. 
 * 
 * Implementa métodos diversos de acceso a base de datos para la entidad Filtro que posibilita
 * obtener listados de Filtros, realizar inserciones de nuevos Filtros, eliminar Filtros etc.
 * 
 * @author Ricardo de los Rios
 * @see ModeloDAO
 */

public class ModeloFiltroDAO extends ModeloDAO{

       // Variables 
       Connection conexion=null; 
       ResultSet resultado=null; 
       private static List<Filtro> listaFiltros;


 /**
 *  Método para obtener un Filtro dado su código.
 * 
 * @param idFiltro Código del Filtro.
 * @return Filtro o null si no existe para ese código.
 */   
 
       public Filtro getFiltro(int idFiltro) 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              Filtro Filtro=null; 
              String consultaString=null;

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // Creacion de la consulta 
                     consultaString = "SELECT * FROM busqueda WHERE idBusqueda=?";

                     // Se prepara la consulta

                     consulta=conexion.prepareStatement(consultaString); 
                     consulta.setInt(1,idFiltro);

                     // Ejecucion de la consulta 
                     resultado=consulta.executeQuery();

                     // Se almacena el resultado en el objeto Filtro 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                   // Se realiza el mapping de los atributos con los campos de la tabla SQL 
                                  Filtro=mapperFiltro(resultado); 
                            } 
                     } 
              } 
              catch(Exception e) 
              { 
                     Filtro=null; 
                     System.out.println("Error en la consulta en la clase ModeloFiltroDAO funcion getFiltro"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                            } 
                            if(consulta!=null) 
                            {
                                consulta.close();
                            } 
                            if(conexion!=null) 
                            {
                               conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexión con la base de datos en la clase ModeloFiltroDAO función getFiltro"); 
                     } 
              }

              // Devolver objeto Filtro 
              return Filtro; 
       }      

 /**
 *  Método para agregar un nuevo Filtro a la base de datos.
 * 
 * @param Filtro Información del Filtro.
 */   

       public void agregarFiltro(Filtro Filtro) throws ExcepcionInmuebles 
       { 
              // Variables 
              PreparedStatement consulta1=null; 
              PreparedStatement consulta2=null; 
              PreparedStatement consulta3=null; 
              PreparedStatement consulta4=null; 

              String consultaString=null; 
              int codigoError=0;
              String idinmueble="";
              String idFiltro="";

              try 
              { 
                     // Apertura de una conexión 
                     conexion=super.getConnection();

                     // Creación del nuevo en inmueble
                     consultaString="INSERT INTO inmueble (tipopiso,superficie, distrito,direccion, codigopostal,"
                             + " orientacion,  numDormitorios, numbanios,"
                             + "ascensor,trastero, terraza, piscina, aguacalienteindv,calefaccionindv, exterior"
                             + " VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

                     consulta1=conexion.prepareStatement(consultaString); 
                     consulta1.setString(1, Filtro.getTipo_inmueble());
                     //consulta1.setString(2, Filtro.getSuperficie());
                     consulta1.setString(3, Filtro.getDistrito());
                     //consulta1.setString(4, Filtro.getDireccion());
                     //consulta1.setString(5, Filtro.getCodigo_postal());
                     consulta1.setString(6, Filtro.getOrientacion());
                     consulta1.setString(7, Filtro.getNum_dorms());
                     consulta1.setString(8, Filtro.getNum_banios());
                     consulta1.setString(9, Filtro.getAscensor());
                     consulta1.setString(10, Filtro.getTrastero());
                     consulta1.setString(11, Filtro.getTerraza());
                     consulta1.setString(12, Filtro.getPiscina());
                     consulta1.setString(13, Filtro.getAgua_caliente());
                     consulta1.setString(14, Filtro.getCalefaccion_indv());
                     consulta1.setString(15, Filtro.getExterior());

                     codigoError=consulta1.executeUpdate();                     

                                          // Recupera el nuevo codigo de inmueble
                     consultaString="select Max(idinmueble) from inmueble";
                     
                    resultado=consulta2.executeQuery();
 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                   // Se realiza el mapping de los atributos con los campos de la tabla SQL 
                                  idinmueble=resultado.getString(1); 

                            } 
                     } 
                     
                     // Creación de la consulta en Filtro
                     consultaString="INSERT INTO Anuncio (inmueble,idusuario, tipoaccion,fechapublicacion, descripcion,"
                             + " precio, estado, alquilergarantizado, VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

                     consulta3=conexion.prepareStatement(consultaString); 
                     consulta3.setString(1, idinmueble);
                     consulta3.setString(2, Filtro.getIdUsuario());
                     consulta3.setString(3, Filtro.getTipoAccion());
                     consulta3.setString(4, Filtro.getFecha_publicacion());
                     //consulta3.setString(5, Filtro.getDescripcion());
                     //consulta3.setString(6, Filtro.getPrecio());
                     consulta3.setString(7, Filtro.getEstado());
                     consulta3.setString(8, Filtro.getAlquiler_garantizado());

                     codigoError=consulta3.executeUpdate(); 

                 // Recupera el nuevo codigo de Filtro
                     consultaString="select Max(idFiltro) from Filtro";
                     
                    resultado=consulta4.executeQuery();
 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                   // Se realiza el mapping de los atributos con los campos de la tabla SQL 
                                  idFiltro=resultado.getString(1); 
                       //           Filtro.setIdFiltro(idFiltro);
                            } 
                     } 

              } 
              catch(SQLException esql) 
              { 
                     
                     System.out.println("Error SQL en la consulta de la clase ModeloFiltroDAO función agregarFiltro" +
                             "\n Flag de executeUpdate: " + codigoError +
                             "\n Codigo de error SQL: " + esql.getErrorCode() +
                             "\n Mensaje de error: " + esql.getMessage() +
                             "\n Estado SQL del error: " + esql.getSQLState()); 
                    
                     throw new ExcepcionInmuebles(ExcepcionInmuebles.EXCEPCION_FUNCIONAL,esql.getMessage()); 
                  
              } 
              catch(Exception e) 
              { 
                  String ra="Error genérico en la consulta de la clase ModeloUsuarioDAO función agregarUsuario" +
                             "\n Codigo de error: " + codigoError +
                             "\n Mensaje de error: " + e.getMessage();          
                  
                  throw new ExcepcionInmuebles(ExcepcionInmuebles.EXCEPCION_TECNICA,ra); 
              }
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                            } 
                            if(consulta1!=null) 
                            {                             
                                consulta1.close(); 
                            } 
                            if(consulta2!=null) 
                            {                             
                                consulta2.close(); 
                            } 
                            if(consulta3!=null)  { consulta3.close(); } 
                            if(consulta4!=null) {  consulta4.close(); } 
                            if(conexion!=null) { conexion.close(); } 

                     } 
                     catch(Exception ex) 
                     { 
                            String ra="Error en el cierre de la conexion con la base de datos en la clase ModeloUsuarioDAO funcion agregarUsuario";                            
                            ra+=ex.getMessage();
                            
                            throw new ExcepcionInmuebles(ExcepcionInmuebles.EXCEPCION_TECNICA,ra);                      } 
                     
                       }
              }

            
    

/**
 *  Método para mapear la información de un Filtro desde un objeto ResultSet de SQL al bean java.
 * 
 * @param resultado ResultSet con información del Filtro
 * @return Bean de java rellenado con la información
 */   
    
    // Realizar el mapping relacional hacia objeto 
    public Filtro mapperFiltro(ResultSet resultado) 
    { 
       // Variables 
       Filtro Filtro=new Filtro();

       try 
       { 
              if (resultado.getString("idFiltro")==null) 
              { 
                     Filtro.setIdFiltro("0"); 
              } 
              else 
              {
                Filtro.setIdFiltro(resultado.getString("idFiltro"));
              }
              
              if (resultado.getString("tipoAccion")==null) 
              { 
                     Filtro.setIdFiltro("0"); 
              } 
              else 
              {
                Filtro.setTipoAccion(resultado.getString("tipoAccion"));
              }

       } 
        catch (Exception e) 
        { 
              //Si se produce un error durante el mapping de atributos 
              System.out.println("Error en el mapping de atributos de un usuario de la clase ModeloUsuarioDAO, funcion mapperUsuario"); 
        }

        // Devolver objeto usuario 
        return Filtro; 
    }

       // eliminar un Filtro en la base 
       public int eliminarFiltro(int idFiltro) 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              String consultaString=null; 
              int codigoError=0;

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // Eliminar el Filtro 
                     consultaString="DELETE FROM Filtro WHERE idFiltro=?";

       consulta=conexion.prepareStatement(consultaString); 
                     consulta.setInt(1,idFiltro);

                     // Ejecucion de la consulta 
                     codigoError=consulta.executeUpdate(); 
              } 
              catch(Exception e) 
              { 
                     codigoError=0; 
                     System.out.println("Error en la consulta de la clase ModeloFiltroDAO funcion eliminarFiltro"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                
                                   resultado.close();
                            } 
                            if(consulta!=null) 
                            {

                                    consulta.close();
                            } 
                            if(conexion!=null) 
                            {

                                conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexion con la base de datos en la clase ModeloFiltroDAO funcion eliminarFiltro"); 
                      } 
              }

              // Devolver el codigo de error 
              return codigoError; 
       }
 
}