package inmuebles.modelos;

import java.sql.Connection; 
import java.sql.PreparedStatement; 
import java.sql.ResultSet; 
import java.sql.SQLException; 

import java.util.ArrayList; 
import java.util.List; 
import inmuebles.beans.Pais;

import inmuebles.util.ExcepcionInmuebles;

/**
 * Clase DAO que obtiene información paises. 
 * 
 * 
 * @author Ricardo de los Rios
 * @see ModeloDAO
 */

public class ModeloPaisDAO extends ModeloDAO{

       // Variables 
       Connection conexion=null; 
       ResultSet resultado=null; 
       private static List<Pais> listaPaises;

/**
 *  Método para obtener la lista de paises.
 * 
 * @return Lista de Paises.
 */   
        
       // devolver la lista de Paises
       public List<Pais> getListaPaises() 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              Pais Pais=null; 
              String consultaString=null; 
              listaPaises=new ArrayList<Pais>();

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // consulta de lista de Paises 
                    consultaString="SELECT * FROM Pais";

                    consulta=conexion.prepareStatement(consultaString);
                    
                     // Ejecución de la consulta 

                    resultado=consulta.executeQuery();


                     // Se almacena el resultado en una lista 
                     if(resultado!=null) 
                     { 
                            while(resultado.next()) 
                            { 
                                   // Se efectua el mapping de los atributos con los campos de la tabla SQL 
                                   Pais=mapperPais(resultado);

                                   // Se aniade el objeto a la lista de Paises

                                    listaPaises.add((Pais)Pais); 
                            } 
                     } 
              } 
              catch(Exception e) 
              { 
                     System.out.println("Error en la consulta de la clase ModeloPaisDAO funcion getListaPaises"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                                  
                            } 
                            if(consulta!=null) 
                            {

                                   consulta.close();
                            } 
                            if(conexion!=null) 
                            {

                                conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexion con la base de datos en la clase ModeloPaisDAO funcion getListaPaises"); 
                     } 
              }

              // Devolver la lista de Paises 
              return listaPaises; 
       }

 /**
 *  Método para obtener un Pais dado su código.
 * 
 * @param idPais Código del Pais.
 * @return Pais o null si no existe para ese código.
 */   
 
       public Pais getPais(int idPais) 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              Pais Pais=null; 
              String consultaString=null;

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // Creacion de la consulta 
                     consultaString = "SELECT * FROM Pais WHERE idPais=?";

                     // Se prepara la consulta

                     consulta=conexion.prepareStatement(consultaString); 
                     consulta.setInt(1,idPais);

                     // Ejecucion de la consulta 
                     resultado=consulta.executeQuery();

                     // Se almacena el resultado en el objeto Pais 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                   // Se realiza el mapping de los atributos con los campos de la tabla SQL 
                                  Pais=mapperPais(resultado); 
                            } 
                     } 
              } 
              catch(Exception e) 
              { 
                     Pais=null; 
                     System.out.println("Error en la consulta en la clase ModeloPaisDAO funcion getPais"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                            } 
                            if(consulta!=null) 
                            {
                                consulta.close();
                            } 
                            if(conexion!=null) 
                            {
                               conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexión con la base de datos en la clase ModeloPaisDAO función getPais"); 
                     } 
              }

              // Devolver objeto Pais 
              return Pais; 
       }      

    

/**
 *  Método para mapear la información de un Pais desde un objeto ResultSet de SQL al bean java.
 * 
 * @param resultado ResultSet con información del Pais
 * @return Bean de java rellenado con la información
 */   
    
    // Realizar el mapping relacional hacia objeto 
    public Pais mapperPais(ResultSet resultado) 
    { 
       // Variables 
       Pais Pais=new Pais();

       try 
       { 
              if (resultado.getString("idPais")==null) 
              { 
                     Pais.setIdPais("0"); 
              } 
              else 
              {
                Pais.setIdPais(resultado.getString("idPais"));
              }
              
              if (resultado.getString("nombre")==null) 
              { 
                     Pais.setIdPais(""); 
              } 
              else 
              {
                Pais.setNombre(resultado.getString("nombre"));
              }

       } 
        catch (Exception e) 
        { 
              //Si se produce un error durante el mapping de atributos 
              System.out.println("Error en el mapping de atributos de un usuario de la clase ModeloUsuarioDAO, funcion mapperUsuario"); 
        }

        // Devolver objeto usuario 
        return Pais; 
    } 
}