package inmuebles.modelos;

import java.sql.Connection; 
import java.sql.PreparedStatement; 
import java.sql.ResultSet; 
import java.sql.SQLException; 

import java.util.ArrayList; 
import java.util.List; 
import inmuebles.beans.Provincia;

import inmuebles.util.ExcepcionInmuebles;

/**
 * Clase DAO que obtiene información de provincias. 
 * 
 * 
 * @author Ricardo de los Rios
 * @see ModeloDAO
 */

public class ModeloProvinciaDAO extends ModeloDAO{

       // Variables 
       Connection conexion=null; 
       ResultSet resultado=null; 
       private static List<Provincia> listaProvincias;

/**
 *  Método para obtener la lista de Provincias.
 * 
 * @return Lista de Provincias.
 */   
        
       // devolver la lista de Provincias
       public List<Provincia> getListaProvincias() 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              Provincia Provincia=null; 
              String consultaString=null; 
              listaProvincias=new ArrayList<Provincia>();

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // consulta de lista de Provincias 
                    consultaString="SELECT * FROM Provincia";

                    consulta=conexion.prepareStatement(consultaString);
                    
                     // Ejecución de la consulta 

                    resultado=consulta.executeQuery();


                     // Se almacena el resultado en una lista 
                     if(resultado!=null) 
                     { 
                            while(resultado.next()) 
                            { 
                                   // Se efectua el mapping de los atributos con los campos de la tabla SQL 
                                   Provincia=mapperProvincia(resultado);

                                   // Se aniade el objeto a la lista de Provincias

                                    listaProvincias.add((Provincia)Provincia); 
                            } 
                     } 
              } 
              catch(Exception e) 
              { 
                     System.out.println("Error en la consulta de la clase ModeloProvinciaDAO funcion getListaProvincias"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                                  
                            } 
                            if(consulta!=null) 
                            {

                                   consulta.close();
                            } 
                            if(conexion!=null) 
                            {

                                conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexion con la base de datos en la clase ModeloProvinciaDAO funcion getListaProvincias"); 
                     } 
              }

              // Devolver la lista de Provincias 
              return listaProvincias; 
       }

 /**
 *  Método para obtener un Provincia dado su código.
 * 
 * @param idProvincia Código del Provincia.
 * @return Provincia o null si no existe para ese código.
 */   
 
       public Provincia getProvincia(int idProvincia) 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              Provincia Provincia=null; 
              String consultaString=null;

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // Creacion de la consulta 
                     consultaString = "SELECT * FROM Provincia WHERE idProvincia=?";

                     // Se prepara la consulta

                     consulta=conexion.prepareStatement(consultaString); 
                     consulta.setInt(1,idProvincia);

                     // Ejecucion de la consulta 
                     resultado=consulta.executeQuery();

                     // Se almacena el resultado en el objeto Provincia 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                   // Se realiza el mapping de los atributos con los campos de la tabla SQL 
                                  Provincia=mapperProvincia(resultado); 
                            } 
                     } 
              } 
              catch(Exception e) 
              { 
                     Provincia=null; 
                     System.out.println("Error en la consulta en la clase ModeloProvinciaDAO funcion getProvincia"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                            } 
                            if(consulta!=null) 
                            {
                                consulta.close();
                            } 
                            if(conexion!=null) 
                            {
                               conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexión con la base de datos en la clase ModeloProvinciaDAO función getProvincia"); 
                     } 
              }

              // Devolver objeto Provincia 
              return Provincia; 
       }      

    

/**
 *  Método para mapear la información de un Provincia desde un objeto ResultSet de SQL al bean java.
 * 
 * @param resultado ResultSet con información del Provincia
 * @return Bean de java rellenado con la información
 */   
    
    // Realizar el mapping relacional hacia objeto 
    public Provincia mapperProvincia(ResultSet resultado) 
    { 
       // Variables 
       Provincia Provincia=new Provincia();

       try 
       { 
              if (resultado.getString("idProvincia")==null) 
              { 
                     Provincia.setIdProvincia("0"); 
              } 
              else 
              {
                Provincia.setIdProvincia(resultado.getString("idProvincia"));
              }
              
              if (resultado.getString("nombre")==null) 
              { 
                     Provincia.setIdProvincia(""); 
              } 
              else 
              {
                Provincia.setNombre(resultado.getString("nombre"));
              }
              
               if (resultado.getString("pais")==null) 
              { 
                     Provincia.setPais(""); 
              } 
              else 
              {
                Provincia.setPais(resultado.getString("pais"));
              }


       } 
        catch (Exception e) 
        { 
              //Si se produce un error durante el mapping de atributos 
              System.out.println("Error en el mapping de atributos de un usuario de la clase ModeloUsuarioDAO, funcion mapperUsuario"); 
        }

        // Devolver objeto usuario 
        return Provincia; 
    } 
}