package inmuebles.modelos;

import java.sql.Connection; 
import java.sql.PreparedStatement; 
import java.sql.ResultSet; 
import java.sql.SQLException; 

import java.util.ArrayList; 
import java.util.List; 
import inmuebles.beans.Usuario;
import inmuebles.util.ExcepcionInmuebles;

/**
 * Clase DAO que representa las operaciones a realizar sobre los usuarios del portal. 
 * 
 * Implementa métodos diversos de acceso a base de datos para la entidad usuario que posibilita
 * obtener listados de usuarios, realizar inserciones de nuevos usuarios, eliminar usuarios etc.
 * 
 * @author Ricardo de los Rios
 * @see ModeloDAO
 */

public class ModeloUsuarioDAO extends ModeloDAO{

       // Variables 
       Connection conexion=null; 
       ResultSet resultado=null; 
       private static List<Usuario> listaUsuarios;

 /**
 *  Método para obtener la lista total de los usuarios.
 * 
 * @return Información de la lista de usuarios.
 */   
       
       public List<Usuario> getListaUsuarios() 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              Usuario usuario=null; 
              String consultaString=null; 
              listaUsuarios=new ArrayList<Usuario>();

              try 
              { 
                     // Apertura de una conexion 
                    conexion=super.getConnection();

                     // consulta de lista de usuarios 
                    consultaString="SELECT idusuario,nombre FROM usuario";

                    consulta=conexion.prepareStatement(consultaString);

                     // Ejecución de la consulta 

                    resultado=consulta.executeQuery();


                     // Se almacena el resultado en una lista 
                     if(resultado!=null) 
                     { 
                            while(resultado.next()) 
                            { 
                                   // Se efectua el mapping de los atributos con los campos de la tabla SQL 
                                   usuario=mapperUsuario(resultado);

                                   // Se anide el objeto a la lista de usuarios

                                    listaUsuarios.add((Usuario)usuario); 
                            } 
                     } 
              } 
              catch(Exception e) 
              { 
                     System.out.println("Error en la consulta de la clase ModeloUsuarioDAO funcion getListaUsuarios"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {

                                  resultado.close();
                            } 
                            if(consulta!=null) 
                            {

                                   consulta.close(); 
                            } 
                            if(conexion!=null) 
                            {

                                conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexion con la base de datos en la clase ModeloUsuarioDAO funcion getListaUsuarios"); 
                     } 
              }

              // Devolver la lista de usuarios 
              return listaUsuarios; 
       }

/**
 *  Método para obtener información de un usuario dados su código.
 * 
 * @param idUsuario Código del usuario.
 * @return Información del usuario
 */   
       public Usuario getUsuario(int idUsuario) 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              Usuario usuario=null; 
              String consultaString=null;

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // Creacion de la consulta 
                     consultaString = "SELECT * FROM usuario WHERE idUsuario=?";

                     // Se prepara la consulta

                     consulta=conexion.prepareStatement(consultaString); 
                     consulta.setInt(1,idUsuario);

                     // Ejecucion de la consulta 
                     resultado=consulta.executeQuery();

                     // Se almacena el resultado en el objeto usuario 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                   // Se realiza el mapping de los atributos con los campos de la tabla SQL 
                                  usuario=mapperUsuario(resultado); 
                            } 
                     } 
              } 
              catch(Exception e) 
              { 
                     usuario=null; 
                     System.out.println("Error en la consulta en la clase ModeloUsuarioDAO funcion getUsuario"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                            } 
                            if(consulta!=null) 
                            {
                                consulta.close(); 
                            } 
                            if(conexion!=null) 
                            {
                               conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexión con la base de datos en la clase ModeloUsuarioDAO función getUsuario"); 
                     } 
              }

              // Devolver objeto usuario 
              return usuario; 
       }

 /**
 *  Método para obtener información de un usuario dado un mail y su password.
 * 
 * @param correo Correo electrónico
 * @param password Contraseña
 * @return Información del usuario o null.
 */   
       public Usuario getUsuarioMail(String correo,String password) throws ExcepcionInmuebles
       { 
              // Variables 
              PreparedStatement consulta=null; 
              Usuario usuario=null; 
              String consultaString=null;

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // Creacion de la consulta 
                     consultaString = "SELECT * FROM usuario WHERE correo=? AND password=?";

                     // Se prepara la consulta

                     consulta=conexion.prepareStatement(consultaString); 
                     consulta.setString(1,correo);
                     consulta.setString(2,password);
                     
                     // Ejecucion de la consulta 
                     resultado=consulta.executeQuery();

                     // Se almacena el resultado en el objeto usuario 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                   // Se realiza el mapping de los atributos con los campos de la tabla SQL 
                                  usuario=mapperUsuario(resultado); 
                            } 
                     } 
              } 
              catch(Exception e) 
              { 
                  String ra="Error genérico en la consulta de la clase ModeloUsuarioDAO función agregarUsuario" +
                          "\n Mensaje de error: " + e.getMessage();          
                  
                  throw new ExcepcionInmuebles(ExcepcionInmuebles.EXCEPCION_TECNICA,ra); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                            } 
                            if(consulta!=null) 
                            {
                                consulta.close(); 
                            } 
                            if(conexion!=null) 
                            {
                               conexion.close();
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                         System.out.println("Error en el cierre de la conexión con la base de datos en la clase ModeloUsuarioDAO función getUsuario"); 
                         throw new ExcepcionInmuebles(ExcepcionInmuebles.EXCEPCION_TECNICA,ex.getMessage()); 
                           
                     } 
              }

              // Devolver objeto usuario 
              return usuario; 
       }

       
       // Agregar un usuario a la base de datos
       // 1. se agrega nuevo usuario
       // 2. se obtiene el nuevo código de usuario

 /**
 *  Método para agregar un nuevo usuario a la base de datos.
 * 
 * Obtiene el código del nuevo usuario
 * 
 * @param usuario Información del nuevo usuario.
 */   
       
       public void agregarUsuario(Usuario usuario) throws ExcepcionInmuebles 
       { 
              // Variables 
              PreparedStatement consulta1=null; 
              PreparedStatement consulta2=null; 
 
              String consultaString=null; 
              int codigoError=0;
              String idusuario="";

              try 
              { 
                     // Apertura de una conexión 
                     conexion=super.getConnection();

                     // Creación del nuevo usuario
                     consultaString="INSERT INTO usuario (nombre,apellidos, telefono1, telefono2,"
                            + " password, correo, tipousuario)"
                             + " VALUES(?,?,?,?,?,?,'N')";

                     consulta1=conexion.prepareStatement(consultaString); 
                     consulta1.setString(1, usuario.getNombre());
                     consulta1.setString(2, usuario.getApellidos());
                     consulta1.setString(3, usuario.getTelefono1());
                     consulta1.setString(4, usuario.getTelefono2());
                     consulta1.setString(5, usuario.getPassword());
                     consulta1.setString(6, usuario.getCorreo());

                     codigoError=consulta1.executeUpdate();                     
                        
                                          // Recupera el nuevo codigo de usuario
                     consultaString="select Max(idusuario) from usuario";
                     consulta2=conexion.prepareStatement(consultaString); 
                   
                    resultado=consulta2.executeQuery();
 
                     if(resultado!=null) 
                     { 
                            if(resultado.next()) 
                            { 
                                   // Se realiza el mapping de los atributos con los campos de la tabla SQL 
                                  idusuario=resultado.getString(1); 
                                  usuario.setIdUsuario(idusuario);

                            } 
                     } 

                     
              } 
              catch(SQLException esql) 
              { 
                    System.out.println("Error SQL en la consulta de la clase ModeloUsuarioDAO función agregarUsuario" +
                             "\n Flag de executeUpdate: " + codigoError +
                             "\n Codigo de error SQL: " + esql.getErrorCode() +
                             "\n Mensaje de error: " + esql.getMessage() +
                             "\n Estado SQL del error: " + esql.getSQLState());
                    
                  throw new ExcepcionInmuebles(ExcepcionInmuebles.EXCEPCION_FUNCIONAL,esql.getMessage()); 
                     
              } 
              catch(Exception e) 
              { 
                  String ra="Error genérico en la consulta de la clase ModeloUsuarioDAO función agregarUsuario" +
                             "\n Codigo de error: " + codigoError +
                             "\n Mensaje de error: " + e.getMessage();          
                  
                  throw new ExcepcionInmuebles(ExcepcionInmuebles.EXCEPCION_TECNICA,ra); 
              }
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                resultado.close();
                            } 
                            if(consulta1!=null) 
                            {                             
                                consulta1.close();
                            } 
                            if(consulta2!=null) 
                            {                             
                                consulta2.close(); 
                            } 
                            if(conexion!=null) 
                            {
                                   conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
 
                            String ra="Error en el cierre de la conexion con la base de datos en la clase ModeloUsuarioDAO funcion agregarUsuario";                            
                            ra+=ex.getMessage();
                            
                            throw new ExcepcionInmuebles(ExcepcionInmuebles.EXCEPCION_TECNICA,ra); 
                     } 
                     
              
              }

    }

 /**
 *  Método para eliminar un usuario de la base de datos.
 * 
 * @param idUsuario Codigo del usuario.
 * @return Codigo de error.
 */   

       // eliminar un usuario en la base 
       public int eliminarUsuario(int idUsuario) 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              String consultaString=null; 
              
             
              int codigoError=0;

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // Eliminar el usuario 
                     consultaString="DELETE FROM usuario WHERE idUsuario=?";

       consulta=conexion.prepareStatement(consultaString); 
                     consulta.setInt(1,idUsuario);

                     // Ejecucion de la consulta 
                     codigoError=consulta.executeUpdate(); 
              } 
              catch(Exception e) 
              { 
                     codigoError=0; 
                     System.out.println("Error en la consulta de la clase ModeloUsuarioDAO funcion eliminarUsuario"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {
                                
                                   resultado.close();
                            } 
                            if(consulta!=null) 
                            {

                                    consulta.close(); 
                            } 
                            if(conexion!=null) 
                            {

                                conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexion con la base de datos en la clase ModeloUsuarioDAO funcion eliminarUsuario"); 
                      } 
              }

              // Devolver el c�digo de error 
              return codigoError; 
       }


       // modificar un usuario en la base 
/*
               
       public int modificarUsuario(Usuario usuario) 
       { 
              // Variables 
              PreparedStatement consulta=null; 
              String consultaString=null; 
              int codigoError=0;

              try 
              { 
                     // Apertura de una conexion 
                     conexion=super.getConnection();

                     // Creacion de la consulta 
                     consultaString="UPDATE usuario set identificador=?,contrasena=? WHERE idUsuario=?";

       consulta=conexion.prepareStatement(consultaString); 
                     consulta.setString(1, 
usuario.getIdentificador()); 
                     consulta.setString(2, usuario.getContrasena()); 
                     consulta.setInt(3, usuario.getIdUsuario());

                     // Se vacia el usuario por seguridad 
                     usuario=null;

                     // Ejecucion de la consulta 
                     codigoError=consulta.executeUpdate(); 
              } 
              catch(Exception e) 
              { 
                     System.out.println("Error en la consulta de la clase ModeloUsuarioDAO funcion modificarUsuario"); 
              } 
              finally 
              { 
                     try 
                     { 
                            // Cierre de la conexion 
                            if(resultado!=null) 
                            {

       resultado.close();
                            } 
                            if(consulta!=null) 
                            {

       consulta.close(); 
                            } 
                            if(conexion!=null) 
                            {

       conexion.close(); 
                            } 
                     } 
                     catch(Exception ex) 
                     { 
                            System.out.println("Error en el cierre de la conexion con la base de datos en la clase ModeloUsuarioDAO funcion modificarUsuario"); 
                      } 
              }

              // Devolver el codigo de error 
              return codigoError; 
       }

*/
       
/**
 *  Método para mapear la información de un usuario desde un objeto ResultSet de SQL al bean java.
 * 
 * @param resultado ResultSet con información del usuario
 * @return Bean de java rellenado con la información
 */   
  
    
    // Realizar el mapping relacional hacia objeto 
    public Usuario mapperUsuario(ResultSet resultado) 
    { 
       // Variables 
       Usuario usuario=new Usuario();

       try 
       { 
              if (resultado.getString("idUsuario")==null) 
              { 
                     usuario.setIdUsuario("0"); 
              } 
              else 
              {

                usuario.setIdUsuario(resultado.getString("idUsuario"));
              }
              
              if (resultado.getString("nombre")==null) 
              { 
                     usuario.setNombre("0"); 
              } 
              else 
              {

                usuario.setNombre(resultado.getString("nombre"));
              }

              if (resultado.getString("tipousuario")==null) 
              { 
                     usuario.setTipoUsuario("N"); 
              } 
              else 
              {
                usuario.setTipoUsuario(resultado.getString("tipousuario"));
              }

               if (resultado.getString("correo")==null) 
                usuario.setCorreo("");        
              else usuario.setCorreo(resultado.getString("correo"));
               
               if (resultado.getString("password")==null) 
                usuario.setPassword("");        
              else usuario.setPassword(resultado.getString("password"));
               
                 if (resultado.getString("apellidos")==null) 
                usuario.setApellidos("");        
              else usuario.setApellidos(resultado.getString("apellidos")); 
                 
                  if (resultado.getString("telefono1")==null) 
                usuario.setTelefono1("");        
              else usuario.setTelefono1(resultado.getString("telefono1")); 
                  
                   if (resultado.getString("telefono2")==null) 
                usuario.setTelefono2("");        
              else usuario.setTelefono2(resultado.getString("telefono2")); 
              
       } 
        catch (Exception e) 
        { 
              //Si se produce un error durante el mapping de atributos 
              usuario=null; 
              System.out.println("Error en el mapping de atributos de un usuario de la clase ModeloUsuarioDAO, funcion mapperUsuario"); 
        }

        // Devolver objeto usuario 
        return usuario; 
    }

}