package inmuebles.util;

import javax.naming.Context; 
import javax.naming.InitialContext; 
import javax.naming.NamingException; 
import javax.servlet.ServletContext; 
import javax.servlet.ServletContextEvent; 
import javax.servlet.ServletContextListener; 
import javax.sql.DataSource;

/**
 * Escuchador que aplica la conexión a la base de datos. Crea un pool de conexiones.
 * 
 * Es configurado en web.xml.
 * 
 * @author Ricardo de los Rios
 * @since 1.6
 */

public class ApplicationListener implements ServletContextListener {
	/** Contexto del servlet */
       private Context context=null;

 /**
 * Método de inicialización del pool de conexiones.
 * 
 * @param servletContextEvent Acceso a los eventos de contexto del servlet
 */   
       public void contextInitialized(ServletContextEvent servletContextEvent) 
       { 
              ServletContext servletContext=servletContextEvent.getServletContext(); 
              String dataSourceJNDI=servletContext.getInitParameter("dataSourceJNDI");

              try 
              { 
                      context=new InitialContext(); 
                      DataSource dataSource=(DataSource)context.lookup(dataSourceJNDI); 
                      if(dataSource==null) 
                      { 
                             System.out.println("No hay DataSource para el proyecto."); 
                      } 
                      else 
                      { 
                              System.out.println("DataSource cargado."); 
                      } 
                      servletContext.setAttribute("dataSource", dataSource); 
               } 
               catch(NamingException e) 
               { 
                   System.out.println("Error al crear el pool. " + e.getMessage());
                      throw new RuntimeException(); 
               } 
               finally 
               { 
                      try 
                      { 
                             //cerrar el contexto 
                             if(context!=null) 
                             { 
                                    context.close(); 
                             } 
                       } 
                       catch(Exception e) 
                       { 
                              System.out.println("Error iniciando pool de conexiones: " + e.getMessage()); 
                       } 
              } 
       }

 /**
 * Función de cierre del contexto de servlet.
 * 
 * @param servletContextEvent Evento de contexto del servlet
 */    
       public void contextDestroyed(ServletContextEvent servletContextEvent) 
       { 
            try 
              { 
                       //cerrar el contexto 
                       if(context!=null) 
                       { 
                              context.close(); 
                       } 
              } 
              catch(Exception e) 
              { 
                     System.out.println("Error al cerrar contexto de servlet: " + e.getMessage()); 
              } 
       } 
}