package inmuebles.util;

/**
 * Clase que representa de forma genérica una excepción del portal Inmuebles.
 * 
 * Distingue entre excepciones técnicas (caida de base de datos, acceso I/O), internas (casos no cubiertos)
 * y funcionales (que no se respeten las reglas de negocio: crear usuario que ya existe).
 * 
 * @author Ricardo de los Rios
 */
public class ExcepcionInmuebles extends Exception {
 /**
 *  Excepcion que afecta a reglas de integridad de los datos.
 */ 
    public static final String EXCEPCION_FUNCIONAL="F";
 /**
 *  Excepcion que indica fallos de entrada/salida o dificultades de acceso a BD, perifericos..
 */ 
    public static final String EXCEPCION_TECNICA="T";

     /**
 *  Tipo de excepción producida.
 */ 
    private String tipoExcepcion="";  // Tecnica, funcional, etc
 /**
 *  Descripción de la excepción.
 */ 
    private String razon="";

 /**
 *  Constructor que admite el tipo de excepción y el mensaje de la excepción para generar una nueva.
 * 
 * @param tipoExcepcion Puede ser EXCEPCION_FUNCIONAL o EXCEPCION_TÉCNICA
 * @param mensaje Mensaje de texto describiendo la excepcion
 */   
    
    public ExcepcionInmuebles(String tipoExcepcion,String mensaje) {
        this.tipoExcepcion=tipoExcepcion;
        this.razon=mensaje;
        
        System.out.println(this.getMensaje());
    }   

 /**
 *  Método que devuelve un mensaje formateado con información de la excepción.
 * 
 * @return Mensaje formateado
 */   
 
    public final String getMensaje() {
        String mensajeFinal="";
        
        if (tipoExcepcion.equalsIgnoreCase(EXCEPCION_FUNCIONAL)) mensajeFinal+="Excepción funcional: \n";
        else if (tipoExcepcion.equalsIgnoreCase(EXCEPCION_TECNICA)) mensajeFinal+="Excepción tecnica: \n";
        //else if (tipoExcepcion.equalsIgnoreCase(EXCEPCION_INTERNA)) mensajeFinal+="Excepción interna: \n";
        else mensajeFinal+="Excepción no clasificada: ";
          
        return mensajeFinal+this.razon;
    }

       
/**
 *  Método que devuelve el tipo de excepción.
 * 
 * @return tipo de excepcion
 * 
 */
    public String getTipoExcepcion() {
        return tipoExcepcion;
    }

/**
 *  Método para indica el tipo de excepcion.
 * 
 * @param tipoExcepcion Tipo de excepcion
 * 
 */
    public void setTipoExcepcion(String tipoExcepcion) {
        this.tipoExcepcion = tipoExcepcion;
    }

  /**
 *  Método que devuelve informacion textual de la excepcion.
 * 
 * @return tipo de excepcion
 * 
 */
    
    public String getRazon() {
        return razon;
    }

 /**
 *  Método para describir el motivo de la excepcion.
 * 
 * @param razon Descripcion de la excepcion
 * 
 */
    
    public void setRazon(String razon) {
        this.razon = razon;
    }   
}
